"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Chase = void 0;
const LightState = require("node-hue-api/lib/model/lightstate/LightState");
const sleep_1 = require("./sleep");
let Service;
let Characteristic;
class Chase {
    constructor(props) {
        this._log = {};
        this._lights = [];
        this._brightness = 0;
        /**
         * Called by homebridge to gather services.
         */
        this.getServices = () => {
            return [this._infoService, this._lightbulbService];
        };
        /**
         * Handler for switch set event
         * @param callback The callback function to call when complete
         */
        this.onPowerSet = (activeState, callback) => __awaiter(this, void 0, void 0, function* () {
            if (this._isActive != activeState) {
                activeState ? this.chase() : this.stop();
            }
            return callback();
        });
        /**
         * Handler for switch get event
         * @param callback The callback function to call when complete
         */
        this.onPowerGet = (callback) => {
            return callback(null, this._isActive);
        };
        this.onBrightnessSet = (newValue, callback) => __awaiter(this, void 0, void 0, function* () {
            this._brightness = newValue;
            const lightState = new LightState();
            lightState.on(true).brightness(this._brightness);
            yield this.setLights(lightState);
            return callback();
        });
        this.onBrightnessGet = (callback) => {
            return callback(null, this._brightness);
        };
        /**
         * Popuplates internal lights array using the configuration values
         */
        this.getLights = () => __awaiter(this, void 0, void 0, function* () {
            //Get lights
            const lightPromises = this._sequence.lights.map((value) => __awaiter(this, void 0, void 0, function* () {
                //@ts-ignore
                const light = yield this._hue.lights.getLightByName(value);
                this._lights.push(light);
            }));
            yield Promise.all(lightPromises);
        });
        /**
         * Execute chase sequence
         */
        this.chase = () => __awaiter(this, void 0, void 0, function* () {
            if (this._lights.length === 0) {
                yield this.getLights();
            }
            const lightState = new LightState();
            let idx = 0;
            this._isActive = true;
            while (this._isActive) {
                const rgb = this.hexToRgb(this._sequence.colors[idx]);
                lightState
                    .on(true)
                    .brightness(this._brightness)
                    .transitionInMillis(this._sequence.transitionTime)
                    .rgb(rgb === null || rgb === void 0 ? void 0 : rgb.red, rgb === null || rgb === void 0 ? void 0 : rgb.green, rgb === null || rgb === void 0 ? void 0 : rgb.blue);
                yield this.setLights(lightState);
                yield (0, sleep_1.Sleep)(this._sequence.transitionTime);
                if (idx == this._sequence.colors.length) {
                    idx = 0;
                }
                else {
                    idx++;
                }
            }
        });
        /**
         * Stop chase sequence
         */
        this.stopAndSetOff = () => __awaiter(this, void 0, void 0, function* () {
            this._isActive = false;
            const lightState = new LightState().off();
            yield this.setLights(lightState);
        });
        this.stop = () => {
            this._isActive = false;
        };
        this.setLights = (state) => __awaiter(this, void 0, void 0, function* () {
            try {
                yield Promise.all(this._lights.map((value) => __awaiter(this, void 0, void 0, function* () {
                    yield this._hue.lights.setLightState(value.id, state);
                })));
            }
            catch (err) {
                this._log(`Error while setting brightness: ${err}`);
            }
        });
        /**
         * Helper function to convert a hex string to rgb
         * @param hex hex string starting with "#"
         */
        this.hexToRgb = (hex) => {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            return result
                ? {
                    red: parseInt(result[1], 16),
                    green: parseInt(result[2], 16),
                    blue: parseInt(result[3], 16),
                }
                : null;
        };
        //Assign class variables
        this._log = props.log;
        this._api = props.api;
        Service = props.api.hap.Service;
        Characteristic = props.api.hap.Characteristic;
        this.name = props.name;
        this._homebridge = props.homebridge;
        this._sequence = props.sequence;
        this._isActive = false;
        this._hue = props.hue;
        this.platformAccessory = new this._homebridge.platformAccessory(this.name, this.generateUUID(), this._homebridge.hap.Accessory.Categories.SWITCH);
        //@ts-ignore
        this._infoService = new Service.AccessoryInformation();
        this._infoService.setCharacteristic(Characteristic.Manufacturer, "Brandon Watson");
        this._infoService.setCharacteristic(Characteristic.Model, "Hue Chase");
        this._infoService.setCharacteristic(Characteristic.SerialNumber, "123-456-789");
        this._lightbulbService = new Service.Lightbulb(this.name, "lightbulbService");
        this._lightbulbService
            .getCharacteristic(Characteristic.On)
            //@ts-ignore
            .on("set", this.onPowerSet)
            //@ts-ignore
            .on("get", this.onPowerGet);
        this._lightbulbService
            .getCharacteristic(Characteristic.Brightness)
            //@ts-ignore
            .on("set", this.onBrightnessSet)
            //@ts-ignore
            .on("get", this.onBrightnessGet);
    }
    /**
     * Helper function to generate a UUID
     */
    generateUUID() {
        // Public Domain/MIT
        var d = new Date().getTime();
        if (typeof performance !== "undefined" &&
            typeof performance.now === "function") {
            d += performance.now(); //use high-precision timer if available
        }
        return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
            var r = (d + Math.random() * 16) % 16 | 0;
            d = Math.floor(d / 16);
            return (c === "x" ? r : (r & 0x3) | 0x8).toString(16);
        });
    }
}
exports.Chase = Chase;
//# sourceMappingURL=chase.js.map