"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chase_1 = require("./chase");
const node_hue_api_1 = require("node-hue-api");
const sleep_1 = require("./sleep");
let Accessory;
let Homebridge;
/**
 * Main entry.
 * @param homebridge
 */
function default_1(homebridge) {
    Homebridge = homebridge;
    Accessory = homebridge.platformAccessory;
    homebridge.registerPlatform("homebridge-hue-chase", "HueChase", HueChasePlatform, true);
}
exports.default = default_1;
class HueChasePlatform {
    constructor(log, config, api) {
        this.log = {};
        this.chaseList = [];
        this.connectHue = () => __awaiter(this, void 0, void 0, function* () {
            if (!this.config) {
                return;
            }
            if (this.config.userName && this.config.clientKey) {
                this.hue = yield node_hue_api_1.v3.api
                    .createLocal(this.config.ipAddress)
                    .connect(this.config.userName, this.config.clientKey, null);
                this.log("Using existing connection info");
            }
            else {
                const unauthenticatedApi = yield node_hue_api_1.v3.api
                    .createLocal(this.config.ipAddress)
                    .connect(null, null, null);
                let createdUser;
                let connected = false;
                while (!connected) {
                    try {
                        this.log("Creating hue user. Push link button");
                        createdUser = yield unauthenticatedApi.users.createUser("homebridge", "HueChase");
                        this.hue = yield node_hue_api_1.v3.api
                            .createLocal(this.config.ipAddress)
                            .connect(createdUser.username, createdUser.clientKey, null);
                        this.log("Connected to Hue Bridge");
                        this.log(`UserName: ${createdUser.username}, ClientKey: ${createdUser.clientkey}`);
                        connected = true;
                    }
                    catch (err) {
                        if (err.getHueErrorType() === 101) {
                            this.log("The Link button on the bridge was not pressed. Please press the Link button and try again.");
                            (0, sleep_1.Sleep)(5000);
                        }
                        else {
                            this.log(`Unexpected Error: ${err.message}`);
                            break;
                        }
                    }
                }
            }
        });
        /**
         * Called by homebridge to gather accessories.
         * @param callback
         */
        this.accessories = (callback) => __awaiter(this, void 0, void 0, function* () {
            //Connect to hue bridge
            yield this.connectHue();
            this.config.sequences.forEach((sequence) => {
                this.chaseList.push(new chase_1.Chase({
                    name: sequence.name,
                    api: this.api,
                    log: this.log,
                    homebridge: Homebridge,
                    sequence: sequence,
                    hue: this.hue,
                }));
            });
            callback(this.chaseList);
        });
        this.log = log;
        this.api = api;
        this.config = config;
        this.log("INFO - Registering Hue Chase platform");
        this.api.on("didFinishLaunching", this.didFinishLaunching.bind(this));
    }
    /**
     * Handler for didFinishLaunching
     * Happens after constructor
     */
    didFinishLaunching() {
        this.log(`INFO - Done registering Hue Chase platform`);
    }
}
//# sourceMappingURL=index.js.map